"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class Flex {
    /**
     * Handles both synchronous and asynchronous values.
     */
    static get(value, options) {
        const onError = options && typeof options.onError === 'function' ? options.onError : null;
        const name = options && typeof options.name === 'string' ? options.name : undefined;
        const cc = options && options.cc;
        const p = value;
        if (p && typeof p.catch === 'function') {
            if (onError) {
                return p.catch(err => onError(err, name));
            }
            return p;
        }
        if (typeof value !== 'function') {
            return value;
        }
        if (!onError) {
            return value.call(cc);
        }
        try {
            const v = value.call(cc);
            if (v && typeof v.catch === 'function') {
                return v.catch(err => onError(err, name));
            }
            return v;
        }
        catch (e) {
            return onError(e, name);
        }
    }
    /**
     * Handles synchronous values, and throws on asynchronous ones.
     */
    static getSync(value, options) {
        const onError = options && typeof options.onError === 'function' ? options.onError : null;
        const name = options && typeof options.name === 'string' ? options.name : undefined;
        const cc = options && options.cc;
        const checkAsync = (v) => {
            if (v && typeof v.then === 'function') {
                const err = new Error(`Value ${name ? '"' + name + '" ' : ''}cannot be asynchronous.`);
                if (onError) {
                    return onError(err, name);
                }
                throw err;
            }
            return v;
        };
        if (typeof value !== 'function') {
            return checkAsync(value);
        }
        const getValue = () => value.call(cc);
        if (!onError) {
            return checkAsync(getValue());
        }
        let v;
        try {
            v = getValue();
        }
        catch (e) {
            return onError(e, name);
        }
        return checkAsync(v);
    }
}
exports.Flex = Flex;
